﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows;
using System.Windows.Media;

namespace xamDataGrid1
{
    public class StandardDeviation : SummaryCalculator
    {
        int _count;
        double _sum;
        double _squaredSum;

        public override string Name
        {
            get
            {
                return "Std. Dev.";
            }
        }

        public override string Description
        {
            get
            {
                return "Standard Deviation";
            }
        }

        public override bool IsCalculationAffectedBySort
        {
            get
            {
                return false;

                //Returning MyBase.IsCalculationAffectedBySort will have the same result as returning False.
            }
        }

        public override ImageSource Image
        {
            get
            {
                ImageSourceConverter converter = new ImageSourceConverter();
                return converter.ConvertFrom("Some_Image_Path.jpg") as ImageSource;
            }
        }

        public override string ApplyDefaultFormat(object summaryResultValue, SummaryResult context)
        {
            return string.Format("{1} = {0:n5}", summaryResultValue, Name);
        }

        public override void BeginCalculation(SummaryResult summaryResult)
        {
            _count = 0;
            _sum = 0;
            _squaredSum = 0;
        }
        public override bool CanProcessDataType(Type dataType)
        {
            return Utilities.IsNumericType(dataType);
        }
        public override void Aggregate(object dataValue, SummaryResult summaryResult, Record record)
        {
            double tempVal = 0.0;

            if (!Convert.IsDBNull(dataValue))
                tempVal = Convert.ToDouble(dataValue);

            _count++;
            _sum += tempVal;
            _squaredSum += tempVal * tempVal;
        }
        public override object EndCalculation(SummaryResult summaryResult)
        {
            if (_count < 2 || _sum == 0)
                return 0;

            double theAverage = _sum / _count;

            double theDeviation = (_squaredSum - _sum * theAverage) / (_count - 1);

            return Math.Sqrt(theDeviation);
        }

    }
}
